@@ -13,15 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from django.conf.urls import url
-from djangosaml2 import views
+from django.urls import path
 
+from . import views
 
 urlpatterns = [
-    url(r'^login/$', views.login, name='saml2_login'),
-    url(r'^acs/$', views.assertion_consumer_service, name='saml2_acs'),
-    url(r'^logout/$', views.logout, name='saml2_logout'),
-    url(r'^ls/$', views.logout_service, name='saml2_ls'),
-    url(r'^ls/post/$', views.logout_service_post, name='saml2_ls_post'),
-    url(r'^metadata/$', views.metadata, name='saml2_metadata'),
+    path('login/', views.login, name='saml2_login'),
+    path('acs/', views.AssertionConsumerServiceView.as_view(), name='saml2_acs'),
+    path('logout/', views.logout, name='saml2_logout'),
+    path('ls/', views.logout_service, name='saml2_ls'),
+    path('ls/post/', views.logout_service_post, name='saml2_ls_post'),
+    path('metadata/', views.metadata, name='saml2_metadata'),
 ]
