@@ -11,8 +11,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import base64
+import re
+import urllib
+import zlib
 
-import django
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.http import is_safe_url
@@ -33,9 +36,12 @@ def available_idps(config, langpref=None):
     for metadata_name, metadata in config.metadata.metadata.items():
         result = metadata.any('idpsso_descriptor', 'single_sign_on_service')
         if result:
-            idps = idps.union(set(result.keys()))
+            idps.update(result.keys())
 
-    return dict([(idp, config.metadata.name(idp, langpref)) for idp in idps])
+    return {
+        idp: config.metadata.name(idp, langpref)
+        for idp in idps
+    }
 
 
 def get_idp_sso_supported_bindings(idp_entity_id=None, config=None):
@@ -43,13 +49,12 @@ def get_idp_sso_supported_bindings(idp_entity_id=None, config=None):
     This is not clear in the pysaml2 code, so wrapping it in a util"""
     if config is None:
         # avoid circular import
-        from djangosaml2.conf import get_config
+        from .conf import get_config
         config = get_config()
     # load metadata store from config
     meta = getattr(config, 'metadata', {})
     # if idp is None, assume only one exists so just use that
     if idp_entity_id is None:
-        # .keys() returns dict_keys in python3.5+
         try:
             idp_entity_id = list(available_idps(config).keys())[0]
         except IndexError:
@@ -82,9 +87,32 @@ def fail_acs_response(request, *args, **kwargs):
     return failure_function(request, *args, **kwargs)
 
 
-def is_safe_url_compat(url, allowed_hosts=None, require_https=False):
-    if django.VERSION >= (1, 11):
-        return is_safe_url(url, allowed_hosts=allowed_hosts, require_https=require_https)
-    assert len(allowed_hosts) == 1
-    host = allowed_hosts.pop()
-    return is_safe_url(url, host=host)
+def validate_referral_url(request, url):
+    # Ensure the user-originating redirection url is safe.
+    # By setting SAML_ALLOWED_HOSTS in settings.py the user may provide a list of "allowed"
+    # hostnames for post-login redirects, much like one would specify ALLOWED_HOSTS .
+    # If this setting is absent, the default is to use the hostname that was used for the current
+    # request.
+    saml_allowed_hosts = set(getattr(settings, 'SAML_ALLOWED_HOSTS', [request.get_host()]))
+
+    if not is_safe_url(url=url, allowed_hosts=saml_allowed_hosts):
+        return settings.LOGIN_REDIRECT_URL
+    else:
+        return url
+
+
+def saml2_from_httpredirect_request(url):
+    urlquery = urllib.parse.urlparse(url).query
+    b64_inflated_saml2req = urllib.parse.parse_qs(urlquery)['SAMLRequest'][0]
+
+    inflated_saml2req = base64.b64decode(b64_inflated_saml2req)
+    deflated_saml2req = zlib.decompress(inflated_saml2req, -15)
+    return deflated_saml2req
+
+def get_session_id_from_saml2(saml2_xml):
+    saml2_xml = saml2_xml.encode() if isinstance(saml2_xml, str) else saml2_xml
+    return re.findall(b'ID="([a-z0-9\-]*)"', saml2_xml, re.I)[0].decode()
+
+def get_subject_id_from_saml2(saml2_xml):
+    saml2_xml = saml2_xml if isinstance(saml2_xml, str) else saml2_xml.decode()
+    re.findall('">([a-z0-9]+)</saml:NameID>', saml2_xml)[0]
