@@ -26,9 +26,11 @@ from django.http import HttpResponseServerError  # 50x
 from django.http import Http404, HttpResponse
 from django.shortcuts import render
 from django.template import TemplateDoesNotExist
+from django.urls import reverse
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic import View
 from django.utils.decorators import method_decorator
+from django.utils.http import urlquote
 
 from saml2 import BINDING_HTTP_REDIRECT, BINDING_HTTP_POST
 from saml2.client_base import LogoutError
@@ -115,7 +117,8 @@ def login(request,
     # is True (default value) we will redirect him to the came_from view.
     # Otherwise, we will show an (configurable) authorization error.
     if request.user.is_authenticated:
-        redirect_authenticated_user = getattr(settings, 'SAML_IGNORE_AUTHENTICATED_USERS_ON_LOGIN', True)
+        redirect_authenticated_user = getattr(settings,
+                                              'SAML_IGNORE_AUTHENTICATED_USERS_ON_LOGIN', True)
         if redirect_authenticated_user:
             return HttpResponseRedirect(came_from)
         else:
@@ -142,10 +145,26 @@ def login(request,
     if getattr(conf, '_sp_allow_create', False):
         kwargs['allow_create'] = "true"
 
+    # Do we have a Discovery Service?
+    discovery_service = getattr(settings, 'SAML2_DISCO_URL', None)
+    if discovery_service and not selected_idp:
+        # We have to build the URL to redirect to with all the information
+        # for the Discovery Service to know how to send the flow back to us
+        logger.debug(("A discovery process is needed trough a"
+                      "Discovery Service: {}").format(discovery_service))
+        login_url = request.build_absolute_uri(reverse('saml2_login'))
+        login_url = '{0}?next={1}'.format(login_url,
+                                          urlquote(came_from, safe=''))
+        ds_url = '{0}?entityID={1}&return={2}&returnIDParam=idp'
+        ds_url = ds_url.format(discovery_service,
+                               urlquote(getattr(conf,'entityid'), safe=''),
+                               urlquote(login_url, safe=''))
+        return HttpResponseRedirect(ds_url)
+
     # is a embedded wayf needed?
     idps = available_idps(conf)
     if selected_idp is None and len(idps) > 1:
-        logger.debug('A discovery process is needed')
+        logger.debug('A discovery process is needed trough WAYF page')
         return render(request, wayf_template, {
                 'available_idps': idps.items(),
                 'came_from': came_from,
@@ -197,7 +216,7 @@ def login(request,
                 kwargs["sigalg"] = sig_alg_option_map[sig_alg_option]
             session_id, result = client.prepare_for_authenticate(
                 entityid=selected_idp, relay_state=came_from,
-                binding=binding, sign=False, nsprefix=nsprefix,
+                binding=binding, sign=sign_requests, nsprefix=nsprefix,
                 **kwargs)
         except TypeError as e:
             logger.error('Unable to know which IdP to use')
@@ -517,7 +536,11 @@ def do_logout_service(request, data, binding, config_loader_path=None, next_page
 
     if 'SAMLResponse' in data:  # we started the logout
         logger.debug('Receiving a logout response from the IdP')
-        response = client.parse_logout_request_response(data['SAMLResponse'], binding)
+        try:
+            response = client.parse_logout_request_response(data['SAMLResponse'], binding)
+        except StatusError as e:
+            response = None
+            logger.warning("Error logging out from remote provider: " + str(e))
         state.sync()
         return finish_logout(request, response, next_page=next_page)
 
@@ -556,7 +579,9 @@ def do_logout_service(request, data, binding, config_loader_path=None, next_page
 
 
 def finish_logout(request, response, next_page=None):
-    if response and response.status_ok():
+    if (getattr(settings, 'SAML_IGNORE_LOGOUT_ERRORS', False) or
+            (response and response.status_ok())):
+
         if next_page is None and hasattr(settings, 'LOGOUT_REDIRECT_URL'):
             next_page = settings.LOGOUT_REDIRECT_URL
         logger.debug('Performing django logout with a next_page of %s',
