@@ -23,6 +23,9 @@ class TestUser(AbstractUser):
     def process_first_name(self, first_name):
         self.first_name = first_name[0]
 
+    class Meta:
+        app_label = 'testprofiles'
+
 
 class StandaloneUserModel(models.Model):
     """
@@ -30,3 +33,16 @@ class StandaloneUserModel(models.Model):
     USERNAME_FIELD.
     """
     username = models.CharField(max_length=30, unique=True)
+
+
+class RequiredFieldUser(models.Model):
+    email = models.EmailField(unique=True)
+    email_verified = models.BooleanField()
+
+    USERNAME_FIELD = 'email'
+
+    def __repr__(self):
+        return self.email
+
+    def set_unusable_password(self):
+        pass
